<?php
namespace Drupal\commerce_pvt\Plugin\views\area;

use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface;
use Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface;
use Drupal\commerce_price\CurrencyFormatter;
use Drupal\commerce_store\CurrentStoreInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\views\Plugin\views\area\AreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines an area handler for the "Summary information" button.
 *
 * @ViewsArea("commerce_pvt_table_summary")
 */
class CommercePVTTableSummary extends AreaPluginBase {

  /**
   * The Order Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface
   */
  protected $chainOrderTypeResolver;

  /**
   * The Order Item Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface
   */
  protected $chainOrderItemTypeResolver;

  /**
   * The Cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The Current store resolved,
   *
   * @var \Drupal\commerce_store\Entity\StoreInterface
   */
  protected $currentStore;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The currency formatter for this class
   *
   * @var \Drupal\commerce_price\CurrencyFormatter
   */
  protected $currencyFormatter;



  /**
   * Constructs a new EmptyCartButton object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface $chain_order_type_resolver
   *   The chain resolver for product order type.
   * @param \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver
   *   The chain resolver for product order item type.
   * @param \Drupal\commerce_cart\CartManagerInterface $cart_manager
   *   The cart manager.
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   The cart provider.
   * @param \Drupal\commerce_store\CurrentStoreInterface $current_store
   *   The current store resolver.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_price\CurrencyFormatter $currency_formatter
   *   The currency formatter.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              ChainOrderTypeResolverInterface $chain_order_type_resolver,
                              ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver,
                              CartManagerInterface $cart_manager,
                              CartProviderInterface $cart_provider,
                              CurrentStoreInterface $current_store,
                              RouteMatchInterface $route_match,
                              EntityTypeManagerInterface $entity_type_manager,
                              CurrencyFormatter $currency_formatter
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->chainOrderTypeResolver = $chain_order_type_resolver;
    $this->chainOrderItemTypeResolver = $chain_order_item_type_resolver;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
    $this->currentStore = $current_store->getStore();
    $this->routeMatch = $route_match;
    $this->entityTypeManager = $entity_type_manager;
    $this->currencyFormatter = $currency_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('commerce_order.chain_order_item_type_resolver'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_store.current_store'),
      $container->get('current_route_match'),
      $container->get('entity_type.manager'),
      $container->get('commerce_price.currency_formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['strip_trailing_zeroes'] = ['default' => 1];
    $options['currency_display'] = ['default' => 'symbol'];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['currency_display'] = [
      '#type' => 'select',
      '#title' => $this->t('Subtotal currency display'),
      '#options' => [
        'symbol' => $this->t('Symbol (e.g. "$")'),
        'code' => $this->t('Currency code (e.g. "USD")'),
        'none' => $this->t('None'),
      ],
      '#default_value' => $this->options['currency_display'],
    ];

    $form['strip_trailing_zeroes'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Strip trailing zeroes after the decimal point.'),
      '#default_value' => $this->options['strip_trailing_zeroes'],
    ];
  }

  /**
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state); // TODO: Change the autogenerated stub

    $options = &$form_state->getValue('options');
    $this->options['currency_display'] = $options['currency_display'];
    $this->options['strip_trailing_zeroes'] = $options['strip_trailing_zeroes'];
  }

  /**
   * Gets whether the views form should be shown when the view has no results.
   *
   * @param bool $empty
   *   Whether the view has results.
   *
   * @return bool
   *   True if the views form should be shown, FALSE otherwise.
   */
  public function viewsFormEmpty($empty) {
    return $empty;
  }

  /**
   * Builds the views form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function viewsForm(array &$form, FormStateInterface $form_state) {

    /** @var \Drupal\commerce_product\Entity\Product $current_product */
    $current_product = $this->routeMatch->getParameter('commerce_product');
    if ($current_product) {
      // Make sure we do not accidentally cache this form.
      $form['#cache']['max-age'] = 0;



      $cart = NULL;
      $default_variation = $current_product->getDefaultVariation();

      if ($default_variation) {
        $order_type = $this->getOrderType($default_variation);

        /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
        if($cart = $this->cartProvider->getCart($order_type, $this->currentStore)) {
          $total_summ = 0;
          $currency_code = NULL;
          foreach ($cart->getItems() as $order_item) {
            $price = $order_item->getTotalPrice();
            if (!$currency_code) {
              $currency_code = $price->getCurrencyCode();
            }
            $total_summ += $price->getNumber();
          }

          $options = $this->getFormatterOptions();

          $markup = new FormattableMarkup('<span class="label">@label</span>: <span class="value">@value</span>', [
            '@label' => $this->t('Product subtotal'),
            '@value' => $this->currencyFormatter->format($total_summ, $currency_code, $options),
          ]);


          $form['#attached']['library'][] = 'commerce_order/total_summary';
          $form['footer'] = [
            '#type' => 'container',
            '#weight' => 100,
          ];

          $form['footer']['summary'] = [
            '#type' => 'item',
            '#markup' => $markup,
            '#prefix' => '<div class="order-total-line">',
            '#suffix' => '</div>',
          ];

          if (isset($form['actions'])) {
            $form['actions']['#weight'] = 200;
          }
        }

      }
    }
  }

  /**
   * Gets the Order Type for which we could take action.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $purcasable_entity
   *
   * @return string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getOrderType($purcasable_entity) {
    $order_item_type = $this->chainOrderItemTypeResolver->resolve($purcasable_entity);
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')
      ->create(['type' => $order_item_type]);
    return $this->chainOrderTypeResolver->resolve($order_item);
  }

  private function getFormatterOptions() {
    $options = [
      'currency_display' => $this->options['currency_display'],
    ];
    if ($this->options['strip_trailing_zeroes']) {
      $options['minimum_fraction_digits'] = 0;
    }

    return $options;
  }

}
