<?php

namespace Drupal\commerce_pvt\Plugin\views\area;

use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface;
use Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface;
use Drupal\commerce_store\CurrentStoreInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\views\Plugin\views\area\AreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines an area handler for the "Empty cart" button.
 *
 * @ViewsArea("commerce_order_pvt_empty_cart_button")
 */
class EmptyCartButton extends AreaPluginBase {

  /**
   * The Order Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface
   */
  protected $chainOrderTypeResolver;

  /**
   * The Order Item Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface
   */
  protected $chainOrderItemTypeResolver;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  /**
   * The Cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The Current store resolved,
   *
   * @var \Drupal\commerce_store\Entity\StoreInterface
   */
  protected $currentStore;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;


  /**
   * Constructs a new EmptyCartButton object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface $chain_order_type_resolver
   *   The chain resolver for product order type.
   * @param \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver
   *   The chain resolver for product order item type.
   * @param \Drupal\commerce_cart\CartManagerInterface $cart_manager
   *   The cart manager.
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   The cart provider.
   * @param \Drupal\commerce_store\CurrentStoreInterface $current_store
   *   The current store resolver.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              ChainOrderTypeResolverInterface $chain_order_type_resolver,
                              ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver,
                              CartManagerInterface $cart_manager,
                              CartProviderInterface $cart_provider,
                              CurrentStoreInterface $current_store,
                              RouteMatchInterface $route_match,
                              EntityTypeManagerInterface $entity_type_manager,
                              MessengerInterface $messenger
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->chainOrderTypeResolver = $chain_order_type_resolver;
    $this->chainOrderItemTypeResolver = $chain_order_item_type_resolver;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
    $this->currentStore = $current_store->getStore();
    $this->routeMatch = $route_match;
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('commerce_order.chain_order_item_type_resolver'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_store.current_store'),
      $container->get('current_route_match'),
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * Gets whether the views form should be shown when the view has no results.
   *
   * @param bool $empty
   *   Whether the view has results.
   *
   * @return bool
   *   True if the views form should be shown, FALSE otherwise.
   */
  public function viewsFormEmpty($empty) {
    return $empty;
  }

  /**
   * Builds the views form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function viewsForm(array &$form, FormStateInterface $form_state) {

    /** @var \Drupal\commerce_product\Entity\Product $current_product */
    $current_product = $this->routeMatch->getParameter('commerce_product');
    if ($current_product) {
      // Make sure we do not accidentally cache this form.
      $form['#cache']['max-age'] = 0;

      $form[$this->options['id']]['#tree'] = TRUE;

      $cart = NULL;
      $default_variation = $current_product->getDefaultVariation();
      if ($default_variation) {
        $order_type = $this->getOrderType($default_variation);

        /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
        $cart = $this->cartProvider->getCart($order_type, $this->currentStore);
        $total_items = 0;
        if ($cart) {
          foreach ($cart->getItems() as $orderItem) {
            $prod_variation = $orderItem->getPurchasedEntity();
            $prod = $prod_variation->getProduct();
            if ($prod->id() == $current_product->id()) {
              $total_items += $orderItem->getQuantity();
            }
          }
        }
      }

      if ($cart && $total_items > 0) {
        $form['actions']['empty_cart'] = [
          '#type' => 'submit',
          '#value' => t('Remove all these variations from cart'),
          '#empty_cart_button' => TRUE,
          '#attributes' => [
            'class' => ['empty-cart-button'],
          ],
        ];

        if (isset($form['actions']['submit']) && !empty($form['actions']['submit']['#ajax'])) {
          $form['#prefix'] = '<div class="commerce-pvt-ajax-form-wrapper">';
          $form['#suffix'] = '</div>';

          $form['actions']['empty_cart']['#ajax'] = [
            'callback' => [get_class($this), 'updatePageElements'],
            'progress' => [
              'type' => 'throbber',
              'message' => $this->t('Removing items from cart').'...',
            ]
          ];
        }
      }
    }
  }

  /**
   * Submits the views form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function viewsFormSubmit(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    if (!empty($triggering_element['#empty_cart_button'])) {

      $cart = NULL;
      $current_product = $this->routeMatch->getParameter('commerce_product');
      $default_variation = $current_product->getDefaultVariation();
      if ($default_variation) {
        $order_type = $this->getOrderType($default_variation);

        /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
        $cart = $this->cartProvider->getCart($order_type, $this->currentStore);
        if ($cart) {
          $save_cart = FALSE;
          foreach ($cart->getItems() as $orderItem) {
            $prod_variation = $orderItem->getPurchasedEntity();
            $prod = $prod_variation->getProduct();
            if ($prod->id() == $current_product->id()) {
              $this->cartManager->removeOrderItem($cart, $orderItem, FALSE);
              $save_cart = TRUE;
            }
          }
          if ($save_cart) {
            if (count($cart->getItems()) == 0) {
              $cart->delete();
            }
            else {
              $cart->save();
            }
            $this->cartProvider->clearCaches();
          }

          if ($this->options['ajax']) {
            $form_state->setRebuild();  // update form after cart emptying.
          }
          else {
            $this->messenger->addMessage($this->t('Your shopping cart has been emptied.'));
          }
        }
      }
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   * @throws \Exception
   */
  public function updatePageElements(array &$form, FormStateInterface $form_state) {
    /** @var RefreshPageElementsHelper $refresher */
    $refresher = \Drupal::service('commerce_pvt.refresh_page_elements_helper');
    return $refresher->updatePageElements($form)->getResponse();
  }

  /**
   * Gets the Order Type for which we could take action.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $purcasable_entity
   *
   * @return string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getOrderType($purcasable_entity) {
    $order_item_type = $this->chainOrderItemTypeResolver->resolve($purcasable_entity);
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')
      ->create(['type' => $order_item_type]);
    return $this->chainOrderTypeResolver->resolve($order_item);
  }

}
