<?php
namespace Drupal\commerce_pvt\Plugin\views\field;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface;
use Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface;
use Drupal\commerce_price\CurrencyFormatter;
use Drupal\commerce_store\CurrentStoreInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Plugin\views\field\UncacheableFieldHandlerTrait;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form element for order item subtotal inside cart
 *
 * @ViewsField("commerce_product_variation_subtotal")
 */
class VariationSubtotal extends FieldPluginBase {

  use UncacheableFieldHandlerTrait;

  /**
   * The Order Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface
   */
  protected $chainOrderTypeResolver;

  /**
   * The Order Item Type Chain resolver.
   *
   * @var \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface
   */
  protected $chainOrderItemTypeResolver;

  /**
   * The Cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The Current store resolved,
   *
   * @var \Drupal\commerce_store\Entity\StoreInterface
   */
  protected $currentStore;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The currency formatter for this class
   *
   * @var \Drupal\commerce_price\CurrencyFormatter
   */
  protected $currencyFormatter;


  /**
   * Constructs a new EditQuantity object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_order\Resolver\ChainOrderTypeResolverInterface $chain_order_type_resolver
   *   The chain resolver for product order type.
   * @param \Drupal\commerce_order\Resolver\ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver
   *   The chain resolver for product order item type.
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   The cart provider.
   * @param \Drupal\commerce_store\CurrentStoreInterface $current_store
   *   The current store resolver.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\commerce_price\CurrencyFormatter $currency_formatter
   *   The currency formatter.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              ChainOrderTypeResolverInterface $chain_order_type_resolver,
                              ChainOrderItemTypeResolverInterface $chain_order_item_type_resolver,
                              CartProviderInterface $cart_provider,
                              CurrentStoreInterface $current_store,
                              RouteMatchInterface $route_match,
                              EntityTypeManagerInterface $entity_type_manager,
                              CurrencyFormatter $currency_formatter
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->chainOrderTypeResolver = $chain_order_type_resolver;
    $this->chainOrderItemTypeResolver = $chain_order_item_type_resolver;
    $this->cartProvider = $cart_provider;
    $this->currentStore = $current_store->getStore();
    $this->routeMatch = $route_match;
    $this->entityTypeManager = $entity_type_manager;
    $this->currencyFormatter = $currency_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('commerce_order.chain_order_item_type_resolver'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_store.current_store'),
      $container->get('current_route_match'),
      $container->get('entity_type.manager'),
      $container->get('commerce_price.currency_formatter')
    );
  }


  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['strip_trailing_zeroes'] = ['default' => 1];
    $options['currency_display'] = ['default' => 'symbol'];

    return $options;
  }


  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    // TODO: make it possible to configure formatters

    $form['currency_display'] = [
      '#type' => 'select',
      '#title' => $this->t('Subtotal currency display'),
      '#options' => [
        'symbol' => $this->t('Symbol (e.g. "$")'),
        'code' => $this->t('Currency code (e.g. "USD")'),
        'none' => $this->t('None'),
      ],
      '#default_value' => $this->options['currency_display'],
    ];

    $form['strip_trailing_zeroes'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Strip trailing zeroes after the decimal point.'),
      '#default_value' => $this->options['strip_trailing_zeroes'],
    ];

  }

  /**
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state); // TODO: Change the autogenerated stub

    $options = &$form_state->getValue('options');
    $this->options['currency_display'] = $options['currency_display'];
    $this->options['strip_trailing_zeroes'] = $options['strip_trailing_zeroes'];
  }

  /**
   * {@inheritdoc}
   */
  public function clickSortable() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getValue(ResultRow $row, $field = NULL) {
    return '<!--form-item-' . $this->options['id'] . '--' . $row->index . '-->';
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function viewsForm(array &$form, FormStateInterface $form_state) {
    $current_product = $this->routeMatch->getParameter('commerce_product');
    if ($current_product) {
      // Make sure we do not accidentally cache this form.
      $form['#cache']['max-age'] = 0;

      $cart = NULL;
      $default_variation = $current_product->getDefaultVariation();
      if ($default_variation) {

        $order_type = $this->getOrderType($default_variation);
        /** @var \Drupal\commerce_order\Entity\OrderInterface $cart */
        if ($cart = $this->cartProvider->getCart($order_type, $this->currentStore)) {

          $form[$this->options['id']]['#tree'] = TRUE;
          foreach ($this->view->result as $row_index => $row) {
            $order_item = $this->getOrderItemFromCart($cart, $row->variation_id);
            $markup = '';
            if ($order_item) {
              $price = $order_item->getTotalPrice();
              $number = $price->getNumber();
              $currency_code  = $price->getCurrencyCode();
              $options = $this->getFormatterOptions();

              $markup = $this->currencyFormatter->format($number, $currency_code, $options);
            }

            $form[$this->options['id']][$row_index] = [
              '#type' => 'item',
              '#markup' => $markup,
            ];
          }

        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function viewsFormSubmit(array &$form, FormStateInterface $form_state) {
    // Do nothing here ...
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing.
  }

  /**
   * @param \Drupal\commerce_order\Entity\OrderInterface $cart
   * @param $variation_id
   *
   * @return \Drupal\commerce_order\Entity\OrderItemInterface|null
   */
  private function getOrderItemFromCart(OrderInterface $cart, $variation_id) {
    if (!$cart) {
      return NULL;
    }
    foreach ($cart->getItems() as $orderItem) {
      if ($orderItem->getPurchasedEntityId() == $variation_id) {
        return $orderItem;
      }
    }
    return NULL;
  }

  /**
   * Gets the Order Type for which we could take action.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $purcasable_entity
   *
   * @return string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getOrderType($purcasable_entity) {
    $order_item_type = $this->chainOrderItemTypeResolver->resolve($purcasable_entity);
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')
      ->create(['type' => $order_item_type]);
    return $this->chainOrderTypeResolver->resolve($order_item);
  }


  private function getFormatterOptions() {
    $options = [
      'currency_display' => $this->options['currency_display'],
    ];
    if ($this->options['strip_trailing_zeroes']) {
      $options['minimum_fraction_digits'] = 0;
    }

    return $options;
  }

}
